@@ -4,33 +4,46 @@ import sys
 from shrine.controllers import SessionRequestHandler
 from shrine.routes import routes
 
+from shrine.conf import settings
 self = sys.modules[__name__]
 
+HTTP_METHODS = ['get', 'post', 'put', 'delete', 'head', 'options', 'patch']
+
 
 class Controller(SessionRequestHandler):
+    def perform(self, method, args, kwargs):
+        if self.requires_authentication and not self.user:
+            if self.request.path != settings.ANONYMOUS_HOME:
+                return self.redirect(settings.ANONYMOUS_HOME)
+            else:
+                return self.finish('You should not be here')
+
+        responder = getattr(self, 'do_{0}'.format(method))
+        return responder(*args, **kwargs)
+
     def get(self, *args, **kw):
-        return self.do_get(*args, **kw)
+        return self.perform('get', args, kw)
 
     def post(self, *args, **kw):
-        return self.do_post(*args, **kw)
+        return self.perform('post', args, kw)
 
     def put(self, *args, **kw):
-        return self.do_put(*args, **kw)
+        return self.perform('put', args, kw)
 
     def delete(self, *args, **kw):
-        return self.do_delete(*args, **kw)
+        return self.perform('delete', args, kw)
 
     def options(self, *args, **kw):
-        return self.do_options(*args, **kw)
+        return self.perform('options', args, kw)
 
     def head(self, *args, **kw):
-        return self.do_head(*args, **kw)
+        return self.perform('head', args, kw)
 
     def patch(self, *args, **kw):
-        return self.do_patch(*args, **kw)
+        return self.perform('patch', args, kw)
 
 
-def make_controller(method, function, pattern, bases):
+def make_controller(method, function, pattern, bases, authenticated=False):
     # `r_` stands for "registered"
     BaseController = Controller
     pops = []
@@ -47,14 +60,14 @@ def make_controller(method, function, pattern, bases):
     name = function.__name__.title() + method.title()
     parents = (BaseController, ) + tuple(bases)
     function_name = 'do_{}'.format(method.lower())
-    return type(name, parents, {function_name: function})
+    return type(name, parents, {function_name: function, 'requires_authentication': authenticated})
 
 
 def make_responder(method, bases1=()):
-    def responder(pattern, bases2=()):
+    def responder(pattern, authenticated=False, bases2=()):
         def dec(func):
             bases = tuple(bases1) + tuple(bases2)
-            ctrl = make_controller(method, func, pattern, bases)
+            ctrl = make_controller(method, func, pattern, bases, authenticated=authenticated)
             routes.append((pattern, ctrl))
 
             return func
@@ -63,7 +76,7 @@ def make_responder(method, bases1=()):
 
 
 def mount_responder_factory(obj, bases=()):
-    for method in 'get', 'post', 'put', 'delete', 'head', 'options', 'patch':
+    for method in HTTP_METHODS:
         setattr(obj, method, make_responder(method, bases))
 
     return obj
